package com.game 
{
	import flash.events.Event;
	import flash.geom.Rectangle;
	//import flash.media.Sound;
	import flash.utils.ByteArray;
	//import flash.events.SampleDataEvent;
	import net.flashpunk.Entity;
	import net.flashpunk.FP;
	import net.flashpunk.graphics.Anim;
	import net.flashpunk.graphics.Canvas;
	import net.flashpunk.graphics.Image;
	import net.flashpunk.graphics.Spritemap;
	import net.flashpunk.graphics.Text;
	import net.flashpunk.Tween;
	import net.flashpunk.utils.Ease;
	import net.flashpunk.Sfx;
	import net.flashpunk.tweens.misc.ColorTween;
	import net.flashpunk.World;
	import net.flashpunk.utils.Input;
	import net.flashpunk.utils.Key;
	
	/**
	 * ...
	 * @author Arthur Zeferino
	 */
	public class MainGame extends World 
	{	
		[Embed(source = '../../../resources/songNotes.txt',mimeType="application/octet-stream")]
		protected static const SONGFILE:Class;
		
		[Embed(source = '../../../resources/background.png')]
		protected static const BGFILE:Class;
		
		[Embed(source = '../../../resources/endbg.png')]
		protected static const ENDBGFILE:Class;
		
		[Embed (source = '../../../resources/lddrumkit.mp3')]
		protected static const DRUMSMP3:Class;
		
		[Embed (source = '../../../resources/staticsound.mp3')]
		protected static const STATICSMP3:Class;				

		[Embed (source = '../../../resources/start.mp3')]
		protected static const STARTSTICKMP3:Class;		
		
		protected var drumsBGM:Sfx;
		protected var stickSfx:Sfx;
		protected var staticSfx:Sfx;
		protected var startTween:Tween;
		
		protected var intro:StoryDisplay;
		protected var ending:StoryDisplay;
		protected var endScreenSpt:Entity;
		protected var readytoReturn:Boolean;
		
		protected var started:Boolean;
		protected var finished:Boolean;
		
		protected var points:int;
		protected var comboCounter:int;
		
		protected var finger:Finger;
		protected var background:Entity;
		protected var crowd:CrowdSprite;
		protected var fretBoard:FretBoard;
		protected var guitarString:GuitarString;
		protected var guitarplayerSpt:GuitarMan;
		protected var fretBlink:Array;
		
		protected var stageNotes:NoteList;
		
		protected var notesHolder:NotesHolder;
		protected var NoteQueue:Array;
		
		protected var songTimer:int;
		protected var notePlayer:NotePlayer;
		
		protected var hud:HUD;
		
		protected var endTween:Tween
		protected var paused:Boolean;
		
		public function MainGame() 
		{
			points = 0;
			comboCounter = 1;
			
			stageNotes = new NoteList();
			NoteQueue = new Array();
			
			started = false;
			finished = false;
			
			setSounds();
			setGraphics();
			
			startStory();
			
			paused = false;
			
			readytoReturn = false;
			
			//loadSong();
			//startSong();
		}
		
		protected function setSounds():void
		{
			stickSfx = new Sfx(STARTSTICKMP3, startShow);
			staticSfx = new Sfx(STATICSMP3, prepareForStart);
			drumsBGM = new Sfx(DRUMSMP3);
		}
		
		protected function prepareForStart():void
		{
			stickSfx.play();
		}
		
		protected function startShow():void
		{
			drumsBGM.loop(0.5);
			started = true;
		}
		
		protected function startStory():void
		{
			intro = new StoryDisplay();
			intro.addLine("-= 3rd Annual Whateverville Guitar Championship =-", "");
			intro.addLine("", "");
			intro.addLine("...", "g0");
			intro.addLine("Hey man I have bad news", "m0");
			intro.addLine("Hey man I have bad news..OMG WHAT HAPPENED TO YOUR HANDS!?", "m1");
			intro.addLine("Well, I fell from some stairs and broke them", "g1");
			intro.addLine("HOW ARE YOU GONNA PLAY?", "m1");
			intro.addLine("Well, I still can move these 2 fingers", "g1");
			intro.addLine("What? Well, it doesn't matter, your guitar has only one string and we can't find more", "m0");
			intro.addLine("Fine by me. A finger and a string is all that I need to rock.", "g0");
			intro.addLine("Woah... but that's like.. the bare minium!", "m1");
			intro.addLine("Yeah, there's the theme.", "g0");
			intro.addLine("What theme?", "m2");
			intro.addLine("Nothing... just gimme my guitar man and let's start this show!", "g1");
			intro.addLine("Ok...", "m2");

			ending = new StoryDisplay();
			ending.addLine("...", "m2");
			ending.addLine("...", "g0");
			ending.addLine("Man that was awful", "m0");
			ending.addLine("...", "g0");
			ending.addLine("I did what I could with my limitations you know?", "g0");
			ending.addLine("Yeah... still awful", "m0");
			ending.addLine("Anyway, thanks for playing! I had fun doing this", "g0");
			ending.addLine("Playing what? You're the one who played!", "m0");
			ending.addLine("Oh... right!", "g2");
			
			this.add(intro);
			
			intro.start();
			
		}
		
		protected function setGraphics():void
		{
			finger = new Finger();
			
			notePlayer = new NotePlayer();
			
			fretBoard = new FretBoard();
			notesHolder = new NotesHolder();
			
			background = new Entity();
			background.graphic = new Image(BGFILE);
			
			guitarString = new GuitarString();
			
			guitarplayerSpt = new GuitarMan();
			
			crowd = new CrowdSprite();
			
			guitarplayerSpt.x = 380;
			guitarplayerSpt.y = 35;
			
			crowd.y = 150;
			
			fretBoard.y = 476;
			notesHolder.y = fretBoard.y - 240;
			guitarString.y = 530;
			finger.y = 455;
			finger.layer = -1000;			
			
			this.add(background);
			this.add(guitarplayerSpt);
			this.add(crowd);
			this.add(notesHolder);
			this.add(fretBoard);
			
			fretBlink = new Array();
			for (var i:int = 0; i < 8; i++)
			{
				fretBlink[i] = new FretBlink(stageNotes.getFretColor(i));
				fretBlink[i].x = 2 + i * 100;
				fretBlink[i].y = fretBoard.y;
				this.add(fretBlink[i]);
			}
			
			this.add(guitarString);
			this.add(finger);
			
			finger.visible = false;
			guitarplayerSpt.visible = false;
			
			hud = new HUD();
			this.add(hud);
			ScoreManager.getInstance().hudPtr = hud;
		}
		
		protected function loadSong():void
		{
			var bytes:ByteArray = new SONGFILE();
			var file:String = bytes.readUTFBytes(bytes.length);
			
			var lines:Array = file.split(/\n/);
			for (var i:int = 0; i < lines.length; i++)
			{
				var nt:Array = lines[i].split("@");
				NoteQueue.push( {fret:nt[0], time:nt[1] } );
			}
		}
		
		protected function startSong():void
		{
			stageNotes.stringPos = guitarString.y+20;
			stageNotes.fretBoard = fretBoard;
			stageNotes.worldPtr = this;
			songTimer = 0;
			
			this.remove(intro);
			
			finger.visible = true;
			guitarplayerSpt.visible = true;	
			
			staticSfx.play();
		}
		
		override public function update():void
		{
			if (started)
			{
				if (Input.mousePressed)
				{
					guitarString.stopVib();
					guitarString.press();
					finger.press();
					notePlayer.stop(20);
				}
				else if (Input.mouseReleased)
				{
					finger.release();
					guitarString.stopVib();
					guitarString.release();
					notePlayer.stop(40);	
				}
				
				if (Input.pressed("strum"))
				{
					strum();
					guitarplayerSpt.strum();
				}
				
				
				finger.x = mouseX;
				if (finger.pressing)
				{
					var beforeFt:uint = getFret();
					finger.x = mouseX;
					var afterFt:uint = getFret();
					
					if (beforeFt != afterFt)
					{
						notePlayer.crossFadeto(beforeFt, afterFt);
					}
				}
				
				guitarplayerSpt.setfret(getFret());
					
				songTimer++;
			
				
				if (NoteQueue.length > 0)
				{
					if (NoteQueue[0].time <= songTimer)
						this.add(stageNotes.addNote(NoteQueue.shift().fret));
				}
				
				if (ScoreManager.getInstance().comboCounter > 10)
					crowd.animate(0);
				else if (ScoreManager.getInstance().comboCounter > 20)
					crowd.animate(1);
				else 
					crowd.stopAnimate();
				
				stageNotes.update();
				notePlayer.update();
				
				if (!finished && stageNotes.empty && NoteQueue.length == 0)
				{
					finished = true;
					endTween = new Tween(100, Tween.ONESHOT, endGame);
					drumsBGM.stop();
					this.addTween(endTween, true);
				}
				
			}
			else if (!finished && !intro.showing && !paused)
			{
				loadSong();
				startSong();
				paused = true;
			}
			
			if (readytoReturn)
			{
				if (Input.mouseReleased)
				{
					FP.world = new StartScreen();
				}
			}
			
			super.update();
		}	
		
		protected function strum():void
		{
			var fret:uint = 0;
			if (finger.pressing)
			{
				fret = getFret();
				guitarString.vibrateAt(finger.x + 50);
				
				if (fret < 0) fret = 0;
				if (fret > 7) fret = 7;
			
				notePlayer.play(fret);				
			}
			else
			{
				guitarString.vibrateAt(0);
				notePlayer.play(8);
			}
			
			
			var score:int = stageNotes.checkHit(fret);

			if (score > 0)
			{
				ScoreManager.getInstance().addPoints(score);
				ScoreManager.getInstance().addCombo();
				blinkFret(fret);
			}
			
		}
		
		protected function getFret():uint
		{
			return Math.floor((finger.x + 50)/ FretBoard.FRETSIZE);
		}
		
		public function blinkFret(fret:int):void
		{
			fretBlink[fret].blink();
		}
		
		public function endGame():void
		{
			started = false;
			
			this.remove(finger);
			this.remove(guitarplayerSpt);
			
			ending.endCallBack = showEndScreen;
			this.add(ending);
			ending.start();
		}
		
		public function showEndScreen():void
		{	
			crowd.crowdSound.play();
			endScreenSpt = new Entity(0, 0, new Image(ENDBGFILE));
			readytoReturn = true;
			this.add(endScreenSpt);
		}
		
	/*	protected function distortSound(event:SampleDataEvent):void
		{
			event.data.writeBytes(getPressedSound(sfxBytes));
		}
		
		protected function getPressedSound(bytes:ByteArray):ByteArray
		{
			var fret:int = getFret();
			var pressedBytes:ByteArray = new ByteArray();
			bytes.position = 0;
			
			while (bytes.bytesAvailable > 0)
			{
				pressedBytes.writeFloat(bytes.readFloat());
				pressedBytes.writeFloat(bytes.readFloat());
				//if (bytes.bytesAvailable > 0)
				//	bytes.position += 2;
			}
			
			return pressedBytes;
			
		}*/
	}

}